@@ -1778,6 +1778,16 @@ public:
         //!
         SCI_ANNOTATIONGETSTYLEOFFSET = 2551,
 
+        SCI_EOLANNOTATIONSETTEXT = 2740,
+        SCI_EOLANNOTATIONGETTEXT = 2741,
+        SCI_EOLANNOTATIONSETSTYLE = 2742,
+        SCI_EOLANNOTATIONGETSTYLE = 2743,
+        SCI_EOLANNOTATIONCLEARALL = 2744,
+        SCI_EOLANNOTATIONSETVISIBLE = 2745,
+        SCI_EOLANNOTATIONGETVISIBLE = 2746,
+        SCI_EOLANNOTATIONSETSTYLEOFFSET = 2747,
+        SCI_EOLANNOTATIONGETSTYLEOFFSET = 2748,
+
         //!
         SCI_RELEASEALLEXTENDEDSTYLES = 2552,
 
@@ -2918,6 +2928,22 @@ public:
         ANNOTATION_INDENTED = 3,
     };
 
+    enum
+    {
+        EOLANNOTATION_HIDDEN = 0x0,
+        EOLANNOTATION_STANDARD = 0x1,
+        EOLANNOTATION_BOXED = 0x2,
+        EOLANNOTATION_STADIUM = 0x100,
+        EOLANNOTATION_FLAT_CIRCLE = 0x101,
+        EOLANNOTATION_ANGLE_CIRCLE = 0x102,
+        EOLANNOTATION_CIRCLE_FLAT = 0x110,
+        EOLANNOTATION_FLATS = 0x111,
+        EOLANNOTATION_ANGLE_FLAT = 0x112,
+        EOLANNOTATION_CIRCLE_ANGLE = 0x120,
+        EOLANNOTATION_FLAT_ANGLE = 0x121,
+        EOLANNOTATION_ANGLES = 0x122,
+    };
+
     enum
     {
         EDGE_NONE = 0,
