@@ -120,6 +120,7 @@ void TabWidgetPrivate::initUI()
 
     tabBar = new TabBar(q);
     symbolBar = new SymbolBar(q);
+    symbolBar->setVisible(false);
 
     mainLayout->addWidget(tabBar, 0, Qt::AlignTop);
     mainLayout->addWidget(symbolBar, 0, Qt::AlignTop);
@@ -141,12 +142,6 @@ void TabWidgetPrivate::initConnection()
     connect(tabBar, &TabBar::closeRequested, q, &TabWidget::closeRequested);
     connect(tabBar, &TabBar::saveFileRequested, q, &TabWidget::saveFile);
 
-    connect(EditorCallProxy::instance(), &EditorCallProxy::reqAddAnnotation, this, &TabWidgetPrivate::handleAddAnnotation);
-    connect(EditorCallProxy::instance(), &EditorCallProxy::reqRemoveAnnotation, this, &TabWidgetPrivate::handleRemoveAnnotation);
-    connect(EditorCallProxy::instance(), &EditorCallProxy::reqClearAllAnnotation, this, &TabWidgetPrivate::handleClearAllAnnotation);
-    connect(EditorCallProxy::instance(), &EditorCallProxy::reqSetLineBackgroundColor, this, &TabWidgetPrivate::handleSetLineBackgroundColor);
-    connect(EditorCallProxy::instance(), &EditorCallProxy::reqResetLineBackground, this, &TabWidgetPrivate::handleResetLineBackground);
-    connect(EditorCallProxy::instance(), &EditorCallProxy::reqClearLineBackground, this, &TabWidgetPrivate::handleClearLineBackground);
     connect(EditorCallProxy::instance(), &EditorCallProxy::reqDoRename, this, &TabWidgetPrivate::handleDoRename);
 }
 
@@ -193,16 +188,20 @@ QWidget *TabWidgetPrivate::createSpaceWidget()
     QGridLayout *gridLayout = new QGridLayout;
     gridLayout->setSpacing(10);
     auto addCommandLine = [this, gridLayout](const QString &id) {
-        auto cmd = ActionManager::getInstance()->command(id);
+        auto cmd = ActionManager::instance()->command(id);
         if (!cmd || !cmd->action())
             return;
 
         int row = gridLayout->rowCount();
         gridLayout->addWidget(new QLabel(cmd->action()->text(), q), row, 0, Qt::AlignRight);
-        gridLayout->addWidget(new KeyLabel(cmd->keySequence().toString(), q), row, 1, Qt::AlignLeft);
+        const auto &keyList = cmd->keySequences();
+        for (int i = 0; i < keyList.size(); ++i) {
+            auto key = keyList[i];
+            gridLayout->addWidget(new KeyLabel(key.toString(), q), row, i + 1, Qt::AlignLeft);
+        }
     };
 
-    addCommandLine("File.Open.File");
+    addCommandLine(A_OPEN_FILE);
     addCommandLine("Find.findInDocument");
     addCommandLine("Editor.close");
     addCommandLine("locator.EnterCommand");
@@ -227,7 +226,7 @@ QWidget *TabWidgetPrivate::createFindPlaceHolder()
     return windowService->createFindPlaceHolder(q, docFind);
 }
 
-TextEditor *TabWidgetPrivate::createEditor(const QString &fileName)
+TextEditor *TabWidgetPrivate::createEditor(const QString &fileName, QsciDocument *doc)
 {
     TextEditor *editor = new TextEditor(q);
     editor->updateLineNumberWidth(false);
@@ -235,18 +234,17 @@ TextEditor *TabWidgetPrivate::createEditor(const QString &fileName)
 
     connect(editor, &TextEditor::zoomValueChanged, q, &TabWidget::zoomValueChanged);
     connect(editor, &TextEditor::cursorRecordChanged, this, &TabWidgetPrivate::onCursorRecordChanged);
-    connect(editor, &TextEditor::fileSaved, tabBar, &TabBar::onFileSaved);
     connect(editor, &TextEditor::requestOpenFiles, this, &TabWidgetPrivate::handleOpenFiles);
     connect(editor, &TextEditor::cursorPositionChanged, symbolBar, &SymbolBar::updateSymbol);
+    connect(editor, &TextEditor::modificationChanged, tabBar, std::bind(&TabBar::onModificationChanged, tabBar, fileName, std::placeholders::_1));
+
+    if (doc) {
+        editor->openFileWithDocument(fileName, *doc);
+    } else {
+        editor->openFile(fileName);
+    }
 
-    editor->setFile(fileName);
     editor->setCursorPosition(0, 0);
-
-    connect(editor, &TextEditor::textChanged, this,
-            [this, fileName] {
-                onFileChanged(fileName);
-            });
-
     editorMng.insert(fileName, editor);
     recentOpenedFiles.prepend(fileName);
 
@@ -387,6 +385,7 @@ void TabWidgetPrivate::onTabClosed(const QString &fileName)
 
     if (editorMng.isEmpty()) {
         symbolBar->clear();
+        symbolBar->setVisible(false);
         q->setSplitButtonVisible(false);
         emit q->closeRequested();
     }
@@ -416,54 +415,6 @@ void TabWidgetPrivate::onCursorRecordChanged(int pos)
         prePosRecord.takeFirst();
 }
 
-void TabWidgetPrivate::onFileChanged(const QString &fileName)
-{
-    auto editor = qobject_cast<TextEditor *>(sender());
-    if (!editor)
-        return;
-
-    // The correct value cannot be immediately get by `editor->isModified()`
-    QTimer::singleShot(50, tabBar, [this, editor, fileName] {
-        tabBar->onFileChanged(fileName, editor->isModified());
-    });
-}
-
-void TabWidgetPrivate::handleAddAnnotation(const QString &fileName, const QString &title, const QString &content, int line, AnnotationType type)
-{
-    if (auto editor = findEditor(fileName))
-        editor->addAnnotation(title, content, line, type);
-}
-
-void TabWidgetPrivate::handleRemoveAnnotation(const QString &fileName, const QString &title)
-{
-    if (auto editor = findEditor(fileName))
-        editor->removeAnnotation(title);
-}
-
-void TabWidgetPrivate::handleClearAllAnnotation(const QString &title)
-{
-    for (auto editor : editorMng)
-        editor->removeAnnotation(title);
-}
-
-void TabWidgetPrivate::handleSetLineBackgroundColor(const QString &fileName, int line, const QColor &color)
-{
-    if (auto editor = findEditor(fileName))
-        editor->setLineBackgroundColor(line, color);
-}
-
-void TabWidgetPrivate::handleResetLineBackground(const QString &fileName, int line)
-{
-    if (auto editor = findEditor(fileName))
-        editor->resetLineBackgroundColor(line);
-}
-
-void TabWidgetPrivate::handleClearLineBackground(const QString &fileName)
-{
-    if (auto editor = findEditor(fileName))
-        editor->clearLineBackgroundColor();
-}
-
 void TabWidgetPrivate::handleDoRename(const newlsp::WorkspaceEdit &info)
 {
     if (info.changes) {
@@ -589,6 +540,41 @@ QStringList TabWidget::openedFiles() const
     return files;
 }
 
+QString TabWidget::rangeText(const QString &fileName, const dpfservice::Edit::Range &range, bool &found)
+{
+    if (auto editor = d->findEditor(fileName)) {
+        found = true;
+        int startPos = editor->positionFromLineIndex(range.start.line,
+                                                     range.start.column == -1
+                                                             ? 0
+                                                             : range.start.column);
+        int endPos = range.end.column == -1
+                ? editor->SendScintilla(TextEditor::SCI_GETLINEENDPOSITION, range.end.line)
+                : editor->positionFromLineIndex(range.end.line, range.end.column);
+        if (startPos == -1 || endPos == -1)
+            return {};
+
+        return editor->text(startPos, endPos);
+    }
+
+    found = false;
+    return {};
+}
+
+Edit::Range TabWidget::selectionRange(const QString &fileName, bool &found)
+{
+    if (auto editor = d->findEditor(fileName)) {
+        found = true;
+        Edit::Range range;
+        editor->getSelection(&range.start.line, &range.start.column,
+                             &range.end.line, &range.end.column);
+        return range;
+    }
+
+    found = false;
+    return {};
+}
+
 void TabWidget::setText(const QString &text)
 {
     if (auto editor = d->currentTextEditor()) {
@@ -616,12 +602,56 @@ void TabWidget::replaceAll(const QString &fileName, const QString &oldText,
         d->docFind->replaceAll(editor, oldText, newText, caseSensitive, wholeWords);
 }
 
-void TabWidget::replaceRange(const QString &fileName, int line, int index, int length, const QString &after)
+void TabWidget::replaceText(const QString &fileName, int line, int index, int length, const QString &after)
 {
     if (auto editor = d->findEditor(fileName))
         editor->replaceRange(line, index, line, index + length, after);
 }
 
+bool TabWidget::replaceRange(const QString &fileName, const dpfservice::Edit::Range &range, const QString &newText)
+{
+    if (auto editor = d->findEditor(fileName)) {
+        // When using the `lineLength()` function to get the length, it is calculated in bytes.
+        // When there are Chinese characters in the text, the length obtained is not correct.
+        auto lineLength = [editor](int line, bool rmN) {
+            auto lineText = editor->text(line);
+            int len = lineText.length();
+            if (rmN && lineText.endsWith('\n'))
+                len -= 1;
+            return len;
+        };
+
+        editor->beginUndoAction();
+        auto lineTextList = newText.split('\n');
+        for (int i = 0; i < lineTextList.size(); ++i) {
+            int line = range.start.line + i;
+            int indexFrom = 0;
+            if (i == 0)
+                indexFrom = range.start.column == -1 ? 0 : range.start.column;
+            int indexTo = lineLength(line, true);
+            if (line == range.end.line && indexTo != range.end.column)
+                indexTo = range.end.column;
+
+            if (i == lineTextList.size() - 1 || line == range.end.line) {
+                indexTo = range.end.column;
+                if (indexTo == -1)
+                    indexTo = lineLength(range.end.line, false);
+                QStringList remainderLines;
+                std::copy(lineTextList.begin() + i, lineTextList.end(), std::back_inserter(remainderLines));
+                editor->replaceRange(line, indexFrom, range.end.line, indexTo, remainderLines.join('\n'));
+                break;
+            } else {
+                editor->replaceRange(line, indexFrom, line, indexTo, lineTextList[i]);
+            }
+        }
+
+        editor->endUndoAction();
+        return true;
+    }
+
+    return false;
+}
+
 void TabWidget::saveAll() const
 {
     for (auto editor : d->editorMng.values()) {
@@ -727,12 +757,6 @@ void TabWidget::undo()
         editor->undo();
 }
 
-void TabWidget::setCompletion(const QString &info, const QIcon &icon, const QKeySequence &key)
-{
-    if (auto editor = d->currentTextEditor())
-        editor->setCompletion(info, icon, key);
-}
-
 void TabWidget::gotoNextPosition()
 {
     if (d->nextPosRecord.isEmpty())
@@ -782,6 +806,66 @@ void TabWidget::gotoPreviousPosition()
     editor->gotoPosition(record.pos);
 }
 
+QString TabWidget::lineText(const QString &fileName, int line)
+{
+    if (auto editor = d->findEditor(fileName))
+        return editor->text(line);
+
+    return "";
+}
+
+bool TabWidget::eOLAnnotate(const QString &fileName, const QString &title, const QString &contents, int line, int type)
+{
+    if (auto editor = d->findEditor(fileName)) {
+        editor->addEOLAnnotation(title, contents, line, type);
+        return true;
+    }
+
+    return false;
+}
+
+bool TabWidget::clearEOLAnnotation(const QString &fileName, const QString &title)
+{
+    if (auto editor = d->findEditor(fileName)) {
+        editor->removeEOLAnnotation(title);
+        return true;
+    }
+
+    return false;
+}
+
+void TabWidget::clearAllEOLAnnotation(const QString &title)
+{
+    for (auto editor : d->editorMng.values())
+        editor->removeEOLAnnotation(title);
+}
+
+bool TabWidget::annotate(const QString &fileName, const QString &title, const QString &contents, int line, int type)
+{
+    if (auto editor = d->findEditor(fileName)) {
+        editor->addAnnotation(title, contents, line, type);
+        return true;
+    }
+
+    return false;
+}
+
+bool TabWidget::clearAnnotation(const QString &fileName, const QString &title)
+{
+    if (auto editor = d->findEditor(fileName)) {
+        editor->removeAnnotation(title);
+        return true;
+    }
+
+    return false;
+}
+
+void TabWidget::clearAllAnnotation(const QString &title)
+{
+    for (auto editor : d->editorMng.values())
+        editor->removeAnnotation(title);
+}
+
 void TabWidget::setEditorCursorPosition(int pos)
 {
     if (auto editor = d->currentTextEditor())
@@ -868,12 +952,79 @@ void TabWidget::updateZoomValue(int value)
     }
 }
 
-QWidget *TabWidget::currentWidget() const
+int TabWidget::backgroundMarkerDefine(const QString &fileName, const QColor &color, int defaultMarker)
+{
+    if (auto editor = d->findEditor(fileName))
+        return editor->backgroundMarkerDefine(color, defaultMarker);
+
+    return -1;
+}
+
+bool TabWidget::setRangeBackgroundColor(const QString &fileName, int startLine, int endLine, int marker)
+{
+    if (auto editor = d->findEditor(fileName)) {
+        editor->setRangeBackgroundColor(startLine, endLine, marker);
+        return true;
+    }
+
+    return false;
+}
+
+Edit::Range TabWidget::getBackgroundRange(const QString &fileName, int marker, bool &found)
+{
+    if (auto editor = d->findEditor(fileName)) {
+        found = true;
+        Edit::Range range;
+        editor->getBackgroundRange(marker, &range.start.line, &range.end.line);
+        return range;
+    }
+
+    found = false;
+    return {};
+}
+
+bool TabWidget::clearAllBackground(const QString &fileName, int marker)
+{
+    if (auto editor = d->findEditor(fileName)) {
+        editor->clearAllBackgroundColor(marker);
+        return true;
+    }
+
+    return false;
+}
+
+void TabWidget::showLineWidget(int line, QWidget *widget)
+{
+    if (auto editor = d->currentTextEditor())
+        editor->showLineWidget(line, widget);
+}
+
+void TabWidget::closeLineWidget()
+{
+    if (auto editor = d->currentTextEditor())
+        editor->closeLineWidget();
+}
+
+void TabWidget::cursorPosition(int *line, int *index)
+{
+    if (!line || !index)
+        return;
+
+    if (auto editor = d->currentTextEditor())
+        editor->getCursorPosition(line, index);
+}
+
+TextEditor *TabWidget::currentEditor() const
 {
     return d->currentTextEditor();
 }
 
-void TabWidget::openFile(const QString &fileName)
+TextEditor *TabWidget::findEditor(const QString &fileName)
+{
+    return d->findEditor(fileName);
+}
+
+void TabWidget::openFile(const QString &fileName, QsciDocument *doc)
 {
     if (!QFile::exists(fileName) || QFileInfo(fileName).isDir())
         return;
@@ -887,8 +1038,9 @@ void TabWidget::openFile(const QString &fileName)
     Inotify::globalInstance()->addPath(fileName);
 
     d->symbolBar->setPath(fileName);
+    d->symbolBar->setVisible(true);
     d->tabBar->setFileName(fileName);
-    TextEditor *editor = d->createEditor(fileName);
+    TextEditor *editor = d->createEditor(fileName, doc);
 
     SymbolWidgetGenerator::instance()->symbolWidget()->setEditor(editor);
     d->editorLayout->addWidget(editor);
