@@ -6,15 +6,18 @@
 #include "private/texteditor_p.h"
 #include "utils/editorutils.h"
 #include "common/common.h"
+#include "common/tooltip/tooltip.h"
 #include "settings/settingsdefine.h"
-#include "services/option/optionutils.h"
+#include "utils/resourcemanager.h"
 
-#include "Qsci/qscistyledtext.h"
+#include "services/editor/editor_define.h"
+
+#include "Qsci/qscidocument.h"
+#include "Qsci/qscilexer.h"
 
 #include <DDialog>
 
 #include <QFile>
-#include <QToolTip>
 #include <QScrollBar>
 #include <QFileDialog>
 #include <QApplication>
@@ -42,7 +45,7 @@ void TextEditor::init()
     connect(this, &TextEditor::cursorPositionChanged, this, &TextEditor::onCursorPositionChanged);
 }
 
-void TextEditor::setFile(const QString &fileName)
+void TextEditor::openFile(const QString &fileName)
 {
     if (d->fileName == fileName)
         return;
@@ -66,6 +69,21 @@ void TextEditor::setFile(const QString &fileName)
     endUndoAction();
 }
 
+void TextEditor::openFileWithDocument(const QString &fileName, const QsciDocument &doc)
+{
+    if (d->fileName == fileName)
+        return;
+
+    d->fileName = fileName;
+    setDocument(doc);
+    d->loadLexer();
+    d->initLanguageClient();
+    d->isAutoCompletionEnabled = true;
+
+    if (isModified())
+        emit modificationChanged(true);
+}
+
 QString TextEditor::getFile() const
 {
     return d->fileName;
@@ -94,7 +112,6 @@ void TextEditor::save()
     file.write(text().toUtf8());
     file.close();
     setModified(false);
-    emit fileSaved(d->fileName);
     editor.fileSaved(d->fileName);
 }
 
@@ -124,7 +141,7 @@ void TextEditor::reload()
     int line = 0, index = 0;
     getCursorPosition(&line, &index);
     const auto &markers = d->allMarkers();
-    
+
     QString text;
     QFile file(d->fileName);
     if (file.open(QFile::OpenModeFlag::ReadOnly)) {
@@ -136,6 +153,7 @@ void TextEditor::reload()
 
     d->setMarkers(markers);
     setCursorPosition(line, index);
+    emit textChanged();
 }
 
 void TextEditor::addBreakpoint(int line, bool enabled)
@@ -148,8 +166,8 @@ void TextEditor::addBreakpoint(int line, bool enabled)
     } else {
         markerAdd(line, TextEditorPrivate::BreakpointDisabled);
     }
-    
-    editor.breakpointAdded(d->fileName, line + 1, enabled);
+
+    editor.breakpointAdded(d->fileName, line, enabled);
 }
 
 void TextEditor::removeBreakpoint(int line)
@@ -159,7 +177,7 @@ void TextEditor::removeBreakpoint(int line)
     else
         markerDelete(line, TextEditorPrivate::BreakpointDisabled);
 
-    editor.breakpointRemoved(d->fileName, line + 1);
+    editor.breakpointRemoved(d->fileName, line);
 }
 
 void TextEditor::toggleBreakpoint()
@@ -184,12 +202,12 @@ void TextEditor::setBreakpointEnabled(int line, bool enabled)
         markerDelete(line, TextEditorPrivate::Breakpoint);
         markerAdd(line, TextEditorPrivate::BreakpointDisabled);
     }
-    editor.breakpointStatusChanged(d->fileName, line + 1, enabled);
+    editor.breakpointStatusChanged(d->fileName, line, enabled);
 }
 
 void TextEditor::setBreakpointCondition(int line)
 {
-    editor.setBreakpointCondition(d->fileName, line + 1);
+    editor.setBreakpointCondition(d->fileName, line);
 }
 
 bool TextEditor::breakpointEnabled(int line)
@@ -315,20 +333,42 @@ int TextEditor::cursorPosition()
     return static_cast<int>(SendScintilla(TextEditor::SCI_GETCURRENTPOS));
 }
 
-void TextEditor::setLineBackgroundColor(int line, const QColor &color)
+int TextEditor::backgroundMarkerDefine(const QColor &color, int defaultMarker)
 {
-    markerAdd(line, TextEditorPrivate::CustomLineBackground);
-    setMarkerBackgroundColor(color, TextEditorPrivate::CustomLineBackground);
+    int marker = markerDefine(Background, defaultMarker);
+    setMarkerBackgroundColor(color, marker);
+    return marker;
 }
 
-void TextEditor::resetLineBackgroundColor(int line)
+void TextEditor::setRangeBackgroundColor(int startLine, int endLine, int marker)
 {
-    markerDelete(line, TextEditorPrivate::CustomLineBackground);
+    startLine = qMax(startLine, 0);
+    endLine = qMin(endLine, lines() - 1);
+    if (startLine > endLine)
+        return;
+
+    d->markerCache.insert(marker, { startLine, endLine });
+    for (; startLine <= endLine; ++startLine) {
+        markerAdd(startLine, marker);
+    }
 }
 
-void TextEditor::clearLineBackgroundColor()
+void TextEditor::getBackgroundRange(int marker, int *startLine, int *endLine)
 {
-    markerDeleteAll(TextEditorPrivate::CustomLineBackground);
+    if (!d->markerCache.contains(marker))
+        return;
+
+    if (!startLine || !endLine)
+        return;
+
+    *startLine = d->markerCache[marker].startLine;
+    *endLine = d->markerCache[marker].endLine;
+}
+
+void TextEditor::clearAllBackgroundColor(int marker)
+{
+    d->markerCache.remove(marker);
+    markerDeleteAll(marker);
 }
 
 void TextEditor::showTips(const QString &tips)
@@ -347,28 +387,44 @@ void TextEditor::showTips(int pos, const QString &tips)
         return;
 
     auto point = pointFromPosition(pos);
-    QToolTip::showText(mapToGlobal(point), tips, this);
+    ToolTip::show(mapToGlobal(point), tips, this);
+}
+
+void TextEditor::showTips(int pos, QWidget *w)
+{
+    if (!hasFocus() || !d->tipsDisplayable)
+        return;
+
+    bool isCtrlPressed = qApp->queryKeyboardModifiers().testFlag(Qt::ControlModifier);
+    if (isCtrlPressed)
+        return;
+
+    auto point = pointFromPosition(pos);
+    ToolTip::show(mapToGlobal(point), w, this);
 }
 
 void TextEditor::cancelTips()
 {
-    QToolTip::hideText();
+    ToolTip::hideImmediately();
 }
 
 void TextEditor::addAnnotation(const QString &title, const QString &content, int line, int type)
 {
     QString typeStr;
     switch (type) {
-    case AnnotationType::NoteAnnotation:
+    case dpfservice::Edit::TipAnnotation:
+        typeStr = "Tip";
+        break;
+    case dpfservice::Edit::NoteAnnotation:
         typeStr = "Note";
         break;
-    case AnnotationType::ErrorAnnotation:
+    case dpfservice::Edit::ErrorAnnotation:
         typeStr = "Error";
         break;
-    case AnnotationType::FatalAnnotation:
+    case dpfservice::Edit::FatalAnnotation:
         typeStr = "Fatal";
         break;
-    case AnnotationType::WarningAnnotation:
+    case dpfservice::Edit::WarningAnnotation:
         typeStr = "Warning";
         break;
     }
@@ -397,6 +453,25 @@ void TextEditor::removeAnnotation(const QString &title)
         clearAnnotations(line);
 }
 
+void TextEditor::addEOLAnnotation(const QString &title, const QString &content, int line, int type)
+{
+    d->eOLAnnotationRecords.insertMulti(title, line);
+    auto style = d->createAnnotationStyle(type);
+    eOLAnnotate(line, content, style);
+}
+
+void TextEditor::removeEOLAnnotation(const QString &title)
+{
+    if (!d->eOLAnnotationRecords.contains(title))
+        return;
+
+    auto lineList = d->eOLAnnotationRecords.values(title);
+    d->eOLAnnotationRecords.remove(title);
+
+    for (int line : lineList)
+        clearEOLAnnotations(line);
+}
+
 void TextEditor::commentOperation()
 {
     QStringList fileCommentSettings = getFileCommentSettings(d->commentSettings);
@@ -411,11 +486,17 @@ void TextEditor::commentOperation()
     if (!hasSelectedText()) {
         int cursorLine = 0, cursorIndex = 0;
         getCursorPosition(&cursorLine, &cursorIndex);
-        int lineLastIndex = lineLength(cursorLine);
+        // The `lineLength` interface gets the byte length
+        // and needs to get the character length
+        const auto &lineText = text(cursorLine);
+        int lineLastIndex = lineText.length();
+        if (lineText.endsWith('\n'))
+            lineLastIndex--;
+
         if (hasUncommentedLines(cursorLine, cursorLine, 0, lineLastIndex, fileCommentSettings))
-            addCommentToSelectedLines(cursorLine, cursorLine, 0, lineLastIndex - 1, fileCommentSettings);
+            addCommentToSelectedLines(cursorLine, cursorLine, 0, lineLastIndex, fileCommentSettings);
         else
-            delCommentToSelectedLines(cursorLine, cursorLine, 0, lineLastIndex - 1, fileCommentSettings);
+            delCommentToSelectedLines(cursorLine, cursorLine, 0, lineLastIndex, fileCommentSettings);
         return;
     }
 
@@ -427,8 +508,9 @@ void TextEditor::commentOperation()
 
 QString TextEditor::getFileType()
 {
+    // Get mimetype using only the file name
     QMimeDatabase mimeDatabase;
-    QString mimeTypeName = mimeDatabase.mimeTypeForFile(d->fileName).name();
+    QString mimeTypeName = mimeDatabase.mimeTypeForFile(d->fileName, QMimeDatabase::MatchExtension).name();
     return mimeTypeName;
 }
 
@@ -570,21 +652,17 @@ int TextEditor::positionFromPoint(int x, int y)
 
 void TextEditor::replaceRange(int lineFrom, int indexFrom, int lineTo, int indexTo, const QString &text)
 {
-    SendScintilla(SCI_CLEARSELECTIONS);
-
-    setSelection(lineFrom, indexFrom, lineTo, indexTo);
-    replaceSelectedText(text);
+    int startPos = positionFromLineIndex(lineFrom, indexFrom);
+    int endPos = positionFromLineIndex(lineTo, indexTo);
+    replaceRange(startPos, endPos, text);
 }
 
 void TextEditor::replaceRange(int startPosition, int endPosition, const QString &text)
 {
-    int startLine = 0, startIndex = 0;
-    int endLine = 0, endIndex = 0;
-
-    lineIndexFromPosition(startPosition, &startLine, &startIndex);
-    lineIndexFromPosition(endPosition, &endLine, &endIndex);
-
-    replaceRange(startLine, startIndex, endLine, endIndex, text);
+    SendScintilla(SCI_CLEARSELECTIONS);
+    SendScintilla(SCI_SETTARGETSTART, startPosition);
+    SendScintilla(SCI_SETTARGETEND, endPosition);
+    SendScintilla(SCI_REPLACETARGET, -1, textAsBytes(text).constData());
 }
 
 void TextEditor::insertText(const QString &text)
@@ -614,7 +692,7 @@ void TextEditor::switchHeaderSource()
 {
     if (!d->languageClient)
         return;
-    
+
     d->languageClient->switchHeaderSource(d->fileName);
 }
 
@@ -622,7 +700,7 @@ void TextEditor::followSymbolUnderCursor()
 {
     if (!d->languageClient)
         return;
-    
+
     d->languageClient->followSymbolUnderCursor();
 }
 
@@ -630,7 +708,7 @@ void TextEditor::findUsage()
 {
     if (!d->languageClient)
         return;
-    
+
     d->languageClient->findUsagesActionTriggered();
 }
 
@@ -638,18 +716,10 @@ void TextEditor::renameSymbol()
 {
     if (!d->languageClient)
         return;
-    
+
     d->languageClient->renameActionTriggered();
 }
 
-void TextEditor::setCompletion(const QString &info, const QIcon &icon, const QKeySequence &key)
-{
-    if (!d->completionWidget)
-        return;
-
-    d->completionWidget->setCompletion(info, icon, key);
-}
-
 QString TextEditor::cursorBeforeText() const
 {
     int pos = d->cursorPosition();
@@ -672,6 +742,32 @@ bool TextEditor::isAutomaticInvocationEnabled() const
     return d->isAutoCompletionEnabled;
 }
 
+bool TextEditor::showLineWidget(int line, QWidget *widget)
+{
+    if (line == 0)
+        line += 1;
+
+    if (line < 0 || line >= lines())
+        return false;
+
+    if (d->lineWidgetContainer->isVisible())
+        closeLineWidget();
+
+    d->showAtLine = line;
+    d->setContainerWidget(widget);
+    return true;
+}
+
+void TextEditor::closeLineWidget()
+{
+    if (!d->lineWidgetContainer->isVisible())
+        return;
+
+    d->lineWidgetContainer->setVisible(false);
+    clearAnnotations(d->showAtLine - 1);
+    editor.inlineWidgetClosed();
+}
+
 void TextEditor::onMarginClicked(int margin, int line, Qt::KeyboardModifiers state)
 {
     Q_UNUSED(state)
@@ -733,6 +829,7 @@ void TextEditor::onCursorPositionChanged(int line, int index)
 {
     Q_UNUSED(line)
 
+    d->delayCursorTimer.start();
     editor.cursorPositionChanged(d->fileName, line, index);
     int pos = positionFromLineIndex(line, index);
 
@@ -748,6 +845,10 @@ void TextEditor::onCursorPositionChanged(int line, int index)
 
 void TextEditor::focusOutEvent(QFocusEvent *event)
 {
+    if (!d->lineWidgetContainer->hasFocus())
+        closeLineWidget();
+
+    d->cancelInlineCompletion();
     Q_EMIT focusOut();
     Q_EMIT followTypeEnd();
     QsciScintilla::focusOutEvent(event);
@@ -755,9 +856,28 @@ void TextEditor::focusOutEvent(QFocusEvent *event)
 
 void TextEditor::keyPressEvent(QKeyEvent *event)
 {
+    if (event->key() == Qt::Key_Tab && d->inlineCompletionCache.first != -1) {
+        d->applyInlineCompletion();
+        auto providerList = ResourceManager::instance()->inlineCompletionProviders();
+        for (const auto provider : providerList)
+            provider->accepted();
+        return;
+    }
+
+    if (event->key() == Qt::Key_Escape && d->lineWidgetContainer->isVisible())
+        return closeLineWidget();
+
     if (d->completionWidget->processKeyPressEvent(event))
         return;
 
+    if (event->key() == Qt::Key_Escape && d->inlineCompletionCache.first != -1) {
+        d->cancelInlineCompletion();
+        auto providerList = ResourceManager::instance()->inlineCompletionProviders();
+        for (const auto provider : providerList)
+            provider->rejected();
+        return;
+    }
+
     QsciScintilla::keyPressEvent(event);
 }
 
@@ -773,6 +893,35 @@ void TextEditor::mouseMoveEvent(QMouseEvent *event)
     QsciScintilla::mouseMoveEvent(event);
 }
 
+void TextEditor::mousePressEvent(QMouseEvent *event)
+{
+    if (event->buttons().testFlag(Qt::LeftButton))
+        d->leftButtonPressed = true;
+
+    QsciScintilla::mousePressEvent(event);
+}
+
+void TextEditor::mouseReleaseEvent(QMouseEvent *event)
+{
+    if (!event->buttons().testFlag(Qt::LeftButton))
+        d->leftButtonPressed = false;
+
+    QsciScintilla::mouseReleaseEvent(event);
+}
+
+bool TextEditor::eventFilter(QObject *obj, QEvent *event)
+{
+    if (obj == d->lineWidgetContainer && event->type() == QEvent::Resize) {
+        d->updateLineWidgetPosition();
+    } else if (obj == d->mainWindow() && event->type() == QEvent::Move) {
+        d->updateLineWidgetPosition();
+    } else if (obj == d->completionWidget && event->type() == QEvent::Hide) {
+        d->updateInlineCompletion();
+    }
+
+    return QsciScintilla::eventFilter(obj, event);
+}
+
 bool TextEditor::event(QEvent *event)
 {
     if (!d)
