@@ -5,6 +5,7 @@
 #include "languageclienthandler.h"
 #include "private/languageclienthandler_p.h"
 #include "gui/texteditor.h"
+#include "gui/private/texteditor_p.h"
 #include "gui/settings/editorsettings.h"
 #include "gui/settings/settingsdefine.h"
 #include "lspclientmanager.h"
@@ -13,15 +14,23 @@
 #include "symbol/symbolmanager.h"
 
 #include "services/project/projectservice.h"
+#include "services/editor/editorservice.h"
+#include "common/tooltip/tooltip.h"
 
 #include "Qsci/qscilexer.h"
 
+#include <cmark.h>
 #include <DApplicationHelper>
+#include <DFrame>
+
 #include <QApplication>
+#include <QTextDocumentFragment>
 
 #include <bitset>
+#include <regex>
 
 DGUI_USE_NAMESPACE
+DWIDGET_USE_NAMESPACE
 
 LanguageClientHandlerPrivate::LanguageClientHandlerPrivate(TextEditor *edit, LanguageClientHandler *qq)
     : q(qq),
@@ -39,7 +48,6 @@ LanguageClientHandlerPrivate::~LanguageClientHandlerPrivate()
 
 void LanguageClientHandlerPrivate::init()
 {
-    diagnosticFormat = "%1\n%2:%3";
     textChangedTimer.setSingleShot(true);
     textChangedTimer.setInterval(200);
 
@@ -71,6 +79,7 @@ void LanguageClientHandlerPrivate::initConnection()
     connect(&textChangedTimer, &QTimer::timeout, this, &LanguageClientHandlerPrivate::delayTextChanged);
     connect(&positionChangedTimer, &QTimer::timeout, this, &LanguageClientHandlerPrivate::delayPositionChanged);
     connect(languageWorker, &LanguageWorker::highlightToken, this, &LanguageClientHandlerPrivate::handleHighlightToken);
+    connect(ToolTip::instance(), &ToolTip::hidden, this, [this] { hoverCache.clean(); });
 }
 
 void LanguageClientHandlerPrivate::initLspConnection()
@@ -138,20 +147,6 @@ void LanguageClientHandlerPrivate::initIndicStyle()
     editor->indicatorDefine(TextEditor::TriangleCharacterIndicator, TextEditor::INDIC_POINTCHARACTER);
 }
 
-QString LanguageClientHandlerPrivate::formatDiagnosticMessage(const QString &message, int type)
-{
-    auto result = message;
-    switch (type) {
-    case AnnotationType::ErrorAnnotation:
-        result = diagnosticFormat.arg("Parse Issue", "Error", result);
-        break;
-    default:
-        break;
-    }
-
-    return result;
-}
-
 bool LanguageClientHandlerPrivate::shouldStartCompletion(const QString &insertedText)
 {
     if (insertedText.isEmpty())
@@ -187,8 +182,8 @@ int LanguageClientHandlerPrivate::wordPostion()
 
 newlsp::Client *LanguageClientHandlerPrivate::getClient()
 {
-    if (prjectKey.isValid())
-        return LSPClientManager::instance()->get(prjectKey);
+    if (projectKey.isValid())
+        return LSPClientManager::instance()->get(projectKey);
 
     auto prjSrv = dpfGetService(dpfservice::ProjectService);
     const auto &filePath = editor->getFile();
@@ -198,25 +193,21 @@ newlsp::Client *LanguageClientHandlerPrivate::getClient()
         if (!files.contains(filePath))
             continue;
 
-        prjectKey.language = prj.language().toStdString();
-        prjectKey.workspace = prj.workspaceFolder().toStdString();
+        projectKey.workspace = prj.workspaceFolder().toStdString();
+        projectKey.outputDirectory = prj.buildFolder().toStdString();
         break;
     }
 
-    if (!prjectKey.isValid()) {
+    if (projectKey.workspace.empty()) {
         auto prj = prjSrv->getActiveProjectInfo();
-        prjectKey.language = prj.language().toStdString();
-        prjectKey.workspace = prj.workspaceFolder().toStdString();
+        projectKey.workspace = prj.workspaceFolder().toStdString();
+        projectKey.outputDirectory = prj.buildFolder().toStdString();
     }
 
-    auto fileLangId = support_file::Language::id(filePath);
-    if (fileLangId != prjectKey.language.c_str()) {
-        fileLangId = support_file::Language::idAlias(fileLangId);
-        if (fileLangId != prjectKey.language.c_str())
-            return nullptr;
-    }
+    auto id = support_file::Language::id(filePath);
+    projectKey.language = support_file::Language::idAlias(id).toStdString();
 
-    return LSPClientManager::instance()->get(prjectKey);
+    return LSPClientManager::instance()->get(projectKey);
 }
 
 void LanguageClientHandlerPrivate::handleDiagnostics(const newlsp::PublishDiagnosticsParams &data)
@@ -242,8 +233,18 @@ void LanguageClientHandlerPrivate::handleDiagnostics(const newlsp::PublishDiagno
             editor->SendScintilla(TextEditor::SCI_INDICSETFORE, TextEditor::INDIC_SQUIGGLE, QColor(Qt::red));
             editor->SendScintilla(TextEditor::SCI_INDICATORFILLRANGE, static_cast<ulong>(startPos), endPos - startPos);
 
-            std::string message = val.message.value();
-            diagnosticCache.append({ startPos, endPos, message.c_str(), AnnotationType::ErrorAnnotation });
+            QString message = val.message.value().c_str();
+            if (val.relatedInformation.has_value()) {
+                message += "\n\n";
+                for (const auto &info : val.relatedInformation.value()) {
+                    QString infoMsg("%1:%2:%3:\nnote:%4");
+                    auto path = QUrl(info.location.uri.c_str()).path();
+                    message += infoMsg.arg(path, QString::number(info.location.range.start.line + 1),
+                                           QString::number(info.location.range.start.character + 1),
+                                           info.message.c_str());
+                }
+            }
+            diagnosticCache.append({ startPos, endPos, message, dpfservice::Edit::ErrorAnnotation });
         }
     }
 }
@@ -283,6 +284,11 @@ void LanguageClientHandlerPrivate::handleShowHoverInfo(const newlsp::Hover &hove
     } else if (newlsp::any_contrast<newlsp::MarkupContent>(hover.contents)) {
         auto markupContent = std::any_cast<newlsp::MarkupContent>(hover.contents);
         showText = markupContent.value;
+        if (markupContent.kind == newlsp::Enum::MarkupKind::get()->Markdown) {
+            char *output = cmark_markdown_to_html(showText.c_str(), showText.size(), CMARK_OPT_DEFAULT);
+            showText = output;
+            free(output);
+        }
     } else if (newlsp::any_contrast<newlsp::MarkedString>(hover.contents)) {
         auto markedString = std::any_cast<newlsp::MarkedString>(hover.contents);
         if (!std::string(markedString).empty()) {
@@ -405,18 +411,6 @@ void LanguageClientHandlerPrivate::handleHoveredStart(int position)
     if (!editor || !getClient())
         return;
 
-    if (!diagnosticCache.isEmpty()) {
-        auto iter = std::find_if(diagnosticCache.begin(), diagnosticCache.end(),
-                                 [position](const DiagnosticCache &cache) {
-                                     return cache.contains(position);
-                                 });
-        if (iter != diagnosticCache.end()) {
-            const auto &msg = formatDiagnosticMessage(iter->message, iter->type);
-            editor->showTips(position, msg);
-            return;
-        }
-    }
-
     hoverCache.setPosition(position);
     auto textRange = hoverCache.getTextRange();
     if (!textRange.isEmpty() && textRange.contaions(position))
@@ -429,6 +423,17 @@ void LanguageClientHandlerPrivate::handleHoveredStart(int position)
         return;
 
     hoverCache.setTextRange(static_cast<int>(startPos), static_cast<int>(endPos));
+    if (!diagnosticCache.isEmpty()) {
+        auto iter = std::find_if(diagnosticCache.begin(), diagnosticCache.end(),
+                                 [position](const DiagnosticCache &cache) {
+                                     return cache.contains(position);
+                                 });
+        if (iter != diagnosticCache.end()) {
+            showDiagnosticTip(position, iter->message);
+            return;
+        }
+    }
+
     lsp::Position pos;
     editor->lineIndexFromPosition(position, &pos.line, &pos.character);
     getClient()->docHoverRequest(editor->getFile(), pos);
@@ -560,6 +565,49 @@ void LanguageClientHandlerPrivate::gotoDefinition()
     }
 }
 
+void LanguageClientHandlerPrivate::showDiagnosticTip(int pos, const QString &message)
+{
+    if (!editor)
+        return;
+
+    QWidget *widget = new QWidget;
+    widget->setAutoFillBackground(true);
+    widget->setForegroundRole(QPalette::BrightText);
+    widget->setBackgroundRole(QPalette::Base);
+    QVBoxLayout *layout = new QVBoxLayout(widget);
+    layout->setContentsMargins(5, 5, 5, 5);
+
+    QLabel *msgLabel = new QLabel(message, widget);
+    layout->addWidget(msgLabel);
+    QString repairMsg = tr("%1: <a href='repair'>Repair with %2</a>");
+
+    // create repair tool
+    auto editSrv = dpfGetService(dpfservice::EditorService);
+    auto repairTools = editSrv->getDiagnosticRepairTool();
+    for (auto iter = repairTools.cbegin(); iter != repairTools.cend(); ++iter) {
+        auto callback = iter.value();
+        QLabel *repairLabel = new QLabel(widget);
+        QString msg = message.mid(0, message.indexOf('\n'));
+        repairLabel->setText(repairMsg.arg(iter.key(), iter.key()));
+        connect(repairLabel, &QLabel::linkActivated, this,
+                [msg, callback, this]() {
+                    QJsonObject info {
+                        { "fileName", editor->getFile() },
+                        { "msg", msg }
+                    };
+
+                    QJsonDocument doc(info);
+                    callback(doc.toJson());
+                    ToolTip::hideImmediately();
+                });
+
+        layout->addWidget(new DHorizontalLine(widget));
+        layout->addWidget(repairLabel);
+    }
+
+    editor->showTips(pos, widget);
+}
+
 void LanguageClientHandlerPrivate::handleSwitchHeaderSource(const QString &file)
 {
     if (file.isEmpty())
