@@ -13,16 +13,9 @@
 
 class AskPageWidget;
 class HistoryListWidget;
-class TranslationPageWidget;
 class CodeGeeXWidget : public DTK_WIDGET_NAMESPACE::DFrame
 {
     Q_OBJECT
-
-    enum pageState : uint8_t {
-        AskPage,
-        TrasnlatePage
-    };
-
 public:
     explicit CodeGeeXWidget(QWidget *parent = nullptr);
 
@@ -30,7 +23,6 @@ public Q_SLOTS:
     void onLoginSuccessed();
     void onLogOut();
     void onNewSessionCreated();
-    void toTranslateCode(const QString &code);
     void onCloseHistoryWidget();
     void onShowHistoryWidget();
 
@@ -40,23 +32,19 @@ protected:
 private:
     void initUI();
     void initConnection();
-    void switchPage(pageState state);
 
     void initAskWidget();
     void initHistoryWidget();
-    void initTabBar();
     void initStackWidget();
     void initAskWidgetConnection();
     void initHistoryWidgetConnection();
 
-    DTK_WIDGET_NAMESPACE::DButtonBox *tabBar { nullptr };
     DTK_WIDGET_NAMESPACE::DStackedWidget *stackWidget { nullptr };
 
     QPropertyAnimation *historyWidgetAnimation { nullptr };
 
     AskPageWidget *askPage { nullptr };
     HistoryListWidget *historyWidget { nullptr };
-    TranslationPageWidget *transPage { nullptr };
 
     bool historyShowed { false };
 };
