@@ -5,8 +5,10 @@
 #ifndef INTROPAGE_H
 #define INTROPAGE_H
 
+#include <DPushButton>
 #include <DWidget>
 #include <DLabel>
+
 #include <QList>
 
 DWIDGET_USE_NAMESPACE
@@ -15,6 +17,20 @@ QT_BEGIN_NAMESPACE
 class QVBoxLayout;
 QT_END_NAMESPACE
 
+class SuggestButton : public DPushButton
+{
+    Q_OBJECT
+public:
+    explicit SuggestButton(const QString &text, const QString &iconName, QWidget *parent = nullptr);
+
+protected:
+    void paintEvent(QPaintEvent *event) override;
+
+private:
+    QString originalText { "" };
+    int minimumWidth = 0;
+};
+
 class IntroPage : public DWidget
 {
     Q_OBJECT
