@@ -19,7 +19,9 @@
 #include "modules/dependencemodule.h"
 #include "locator/locatormanager.h"
 #include "find/placeholdermanager.h"
+
 #include "common/util/utils.h"
+#include "common/actionmanager/actioncontainer.h"
 
 #include <DFrame>
 #include <DFileDialog>
@@ -53,9 +55,9 @@ DWIDGET_USE_NAMESPACE
 struct WidgetInfo
 {
     QString name;
-    DWidget *widget { nullptr };
     QDockWidget *dockWidget { nullptr };
     QString headerName;
+    bool showHeader { true };
     QList<QAction *> headerList;
     QList<QWidget *> headerWidget;   // set button to header after create dock
     QIcon icon;
@@ -65,6 +67,28 @@ struct WidgetInfo
     bool defaultVisible { true };
     bool created { false };   // has already create dock
     bool hiddenByManual { false };
+    
+    std::function<AbstractWidget*()> createDWidgetFunc;
+
+    DWidget* getDWidget()
+    {
+        if (!widget) {
+            if (createDWidgetFunc) {
+                auto abstractWidget = createDWidgetFunc(); 
+                icon = abstractWidget->getDisplayIcon();
+                widget = static_cast<DWidget *>(abstractWidget->qWidget());
+                Q_ASSERT(widget);
+                if (!widget->parent())
+                    widget->setParent(Controller::instance()->mainWindow());
+            }
+        }
+        return widget;
+    }
+    
+    void setWidget(DWidget *widget)
+    {
+        this->widget = widget;
+    }
 
     bool operator==(const WidgetInfo &info)
     {
@@ -72,6 +96,9 @@ struct WidgetInfo
             return true;
         return false;
     };
+
+private:    
+    DWidget *widget { nullptr };
 };
 
 class DocksManagerButton : public DToolButton
@@ -95,7 +122,8 @@ class ControllerPrivate
 {
     MainWindow *mainWindow { nullptr };
     loadingWidget *loadingwidget { nullptr };
-    WorkspaceWidget *workspace { nullptr };
+    WorkspaceWidget *workspace { nullptr }; // left default dock widget
+    WorkspaceWidget *rightspace { nullptr }; // right default dock widget
 
     DWidget *navigationToolBar { nullptr };
     NavigationBar *navigationBar { nullptr };
@@ -117,8 +145,8 @@ class ControllerPrivate
     QHBoxLayout *contextButtonLayout { nullptr };
 
     WindowStatusBar *statusBar { nullptr };
-
-    DMenu *menu { nullptr };
+    DToolButton *showContextBtn { nullptr };
+    DToolButton *showRightspaceBtn { nullptr };
 
     QStringList validModeList { CM_EDIT, CM_DEBUG, CM_RECENT };
     QMap<QString, QString> modePluginMap { { CM_EDIT, MWNA_EDIT }, { CM_RECENT, MWNA_RECENT }, { CM_DEBUG, MWNA_DEBUG } };
@@ -158,6 +186,7 @@ Controller::Controller(QObject *parent)
     initMainWindow();
     initNavigationBar();
     initStatusBar();
+    initRightspaceWidget();
     initContextWidget();
     initWorkspaceWidget();
     initTopToolBar();
@@ -196,6 +225,9 @@ void Controller::registerService()
     if (!windowService->registerWidget) {
         windowService->registerWidget = std::bind(&Controller::registerWidget, this, _1, _2);
     }
+    if (!windowService->registerWidgetCreator) {
+        windowService->registerWidgetCreator = std::bind(&Controller::registerWidgetCreator, this, _1, _2);
+    }
     if (!windowService->showWidgetAtPosition) {
         windowService->showWidgetAtPosition = std::bind(&Controller::showWidgetAtPosition, this, _1, _2, _3);
     }
@@ -206,7 +238,7 @@ void Controller::registerService()
         windowService->setDockHeaderList = std::bind(&Controller::setDockHeaderList, this, _1, _2);
     }
     if (!windowService->deleteDockHeader) {
-        windowService->deleteDockHeader = std::bind(&MainWindow::deleteDockHeader, d->mainWindow, _1);
+        windowService->deleteDockHeader = std::bind(&Controller::deleteDockHeader, this, _1);
     }
     if (!windowService->addToolBtnToDockHeader) {
         windowService->addToolBtnToDockHeader = std::bind(&MainWindow::addWidgetToDockHeader, d->mainWindow, _1, _2);
@@ -250,18 +282,6 @@ void Controller::registerService()
     if (!windowService->switchContextWidget) {
         windowService->switchContextWidget = std::bind(&Controller::switchContextWidget, this, _1);
     }
-    if (!windowService->addChildMenu) {
-        windowService->addChildMenu = std::bind(&Controller::addChildMenu, this, _1);
-    }
-    if (!windowService->insertAction) {
-        windowService->insertAction = std::bind(&Controller::insertAction, this, _1, _2, _3);
-    }
-    if (!windowService->addAction) {
-        windowService->addAction = std::bind(&Controller::addAction, this, _1, _2);
-    }
-    if (!windowService->removeActions) {
-        windowService->removeActions = std::bind(&Controller::removeActions, this, _1);
-    }
     if (!windowService->addWidgetToTopTool) {
         windowService->addWidgetToTopTool = std::bind(&Controller::addWidgetToTopTool, this, _1, _2, _3, _4);
     }
@@ -295,9 +315,18 @@ void Controller::registerService()
     if (!windowService->addWidgetWorkspace) {
         windowService->addWidgetWorkspace = std::bind(&WorkspaceWidget::addWorkspaceWidget, d->workspace, _1, _2, _3);
     }
+    if (!windowService->addWidgetRightspace) {
+        windowService->addWidgetRightspace = std::bind(&WorkspaceWidget::addWorkspaceWidget, d->rightspace, _1, _2, _3);
+    }
     if (!windowService->registerToolBtnToWorkspaceWidget) {
         windowService->registerToolBtnToWorkspaceWidget = std::bind(&WorkspaceWidget::registerToolBtnToWidget, d->workspace, _1, _2);
     }
+    if (!windowService->registerToolBtnToRightspaceWidget) {
+        windowService->registerToolBtnToRightspaceWidget = std::bind(&WorkspaceWidget::registerToolBtnToWidget, d->rightspace, _1, _2);
+    }
+    if (!windowService->showWidgetAtRightspace) {
+        windowService->showWidgetAtRightspace = std::bind(&WorkspaceWidget::switchWidgetWorkspace, d->rightspace, _1);
+    }
     if (!windowService->registerWidgetToDockHeader) {
         windowService->registerWidgetToDockHeader = std::bind(&Controller::registerWidgetToDockHeader, this, _1, _2);
     }
@@ -329,7 +358,7 @@ Controller::~Controller()
 
 void Controller::createDockWidget(WidgetInfo &info)
 {
-    auto dock = d->mainWindow->addWidget(info.name, info.widget, info.defaultPos);
+    auto dock = d->mainWindow->addWidget(info.name, info.getDWidget(), info.defaultPos);
     info.dockWidget = dock;
     info.created = true;
 
@@ -338,7 +367,9 @@ void Controller::createDockWidget(WidgetInfo &info)
     else if (!info.headerList.isEmpty())
         d->mainWindow->setDockHeaderList(info.name, info.headerList);
 
-    if (!info.headerWidget.isEmpty()) {
+    if (!info.showHeader) {
+        d->mainWindow->deleteDockHeader(info.name);
+    } else if (!info.headerWidget.isEmpty()) {
         for (auto btn : info.headerWidget)
             d->mainWindow->addWidgetToDockHeader(info.name, btn);
     }
@@ -362,7 +393,7 @@ void Controller::raiseMode(const QString &mode)
         if (!widgetInfo.hiddenByManual)
             d->mainWindow->showWidget(widgetInfo.name);
         // widget in mainWindow is Dock(widget), show dock and hide widget
-        widgetInfo.widget->setVisible(widgetInfo.defaultVisible);
+        widgetInfo.getDWidget()->setVisible(widgetInfo.defaultVisible);
         if (widgetInfo.dockWidget)
             d->currentDocks.append(widgetInfo.name);
     }
@@ -382,6 +413,7 @@ void Controller::raiseMode(const QString &mode)
 
     showTopToolBar();
     showContextWidget();
+    showRightspace();
 
     d->mode = mode;
     uiController.modeRaised(mode);
@@ -394,14 +426,14 @@ void Controller::replaceWidget(const QString &name, Position pos)
 
 void Controller::insertWidget(const QString &name, Position pos, Qt::Orientation orientation)
 {
-    if (d->allWidgets.contains(name)) {
+    if (!d->allWidgets.contains(name)) {
         qWarning() << "no widget named:" << name;
         return;
     }
 
     auto &info = d->allWidgets[name];
     if (!info.created) {
-        auto dock = d->mainWindow->addWidget(name, info.widget, pos, orientation);
+        auto dock = d->mainWindow->addWidget(name, info.getDWidget(), pos, orientation);
         info.dockWidget = dock;
         info.created = true;
         info.replace = false;
@@ -442,7 +474,7 @@ void Controller::registerWidgetToMode(const QString &name, AbstractWidget *abstr
     widgetInfo.name = name;
     widgetInfo.defaultPos = pos;
     widgetInfo.replace = replace;
-    widgetInfo.widget = qWidget;
+    widgetInfo.setWidget(qWidget);
     widgetInfo.defaultVisible = isVisible;
     widgetInfo.icon = abstractWidget->getDisplayIcon();
 
@@ -464,12 +496,24 @@ void Controller::registerWidget(const QString &name, AbstractWidget *abstractWid
 
     WidgetInfo widgetInfo;
     widgetInfo.name = name;
-    widgetInfo.widget = widget;
+    widgetInfo.setWidget(widget);
     widgetInfo.icon = abstractWidget->getDisplayIcon();
 
     d->allWidgets.insert(name, widgetInfo);
 }
 
+void Controller::registerWidgetCreator(const QString &name, std::function<AbstractWidget*()> &widgetCreateFunc)
+{
+     if (d->allWidgets.contains(name))
+        return;
+
+    WidgetInfo widgetInfo;
+    widgetInfo.name = name;
+    widgetInfo.createDWidgetFunc = widgetCreateFunc;
+
+    d->allWidgets.insert(name, widgetInfo);
+}
+
 void Controller::showWidgetAtPosition(const QString &name, Position pos, bool replace)
 {
     if (replace)
@@ -531,6 +575,19 @@ void Controller::setDockHeaderName(const QString &dockName, const QString &heade
         d->mainWindow->setDockHeaderName(dockName, headerName);
 }
 
+void Controller::deleteDockHeader(const QString &dockName)
+{
+    if (!d->allWidgets.contains(dockName)) {
+        qWarning() << "No widget named: " << dockName;
+        return;
+    }
+    auto &info = d->allWidgets[dockName];
+    if (info.created)
+        d->mainWindow->deleteDockHeader(dockName);
+    else
+        info.showHeader = false;
+}
+
 void Controller::setDockHeaderList(const QString &dockName, const QList<QAction *> &actions)
 {
     if (!d->allWidgets.contains(dockName)) {
@@ -668,6 +725,38 @@ void Controller::showContextWidget()
     d->currentDocks.append(WN_CONTEXTWIDGET);
 }
 
+void Controller::showRightspace()
+{
+    auto &rightSpaceInfo = d->allWidgets[WN_RIGHTSPACE];
+    if (!rightSpaceInfo.created) {
+        createDockWidget(d->allWidgets[WN_RIGHTSPACE]);
+        d->mainWindow->showWidget(WN_RIGHTSPACE);
+        d->mainWindow->resizeDock(WN_RIGHTSPACE, QSize(300, 300));
+
+        for (auto btn : d->rightspace->getAllToolBtn())
+            d->mainWindow->addWidgetToDockHeader(WN_RIGHTSPACE, btn);
+
+        auto titles = d->rightspace->allWidgetTitles();
+        QList<QAction *> headers;
+        for (auto title : titles) {
+            QAction *action = new QAction(title, d->rightspace);
+            connect(action, &QAction::triggered, this, [=]() { d->rightspace->switchWidgetWorkspace(title); });
+            headers.append(action);
+        }
+        d->mainWindow->setDockHeaderList(WN_RIGHTSPACE, headers);
+        d->mainWindow->setDockHeaderName(WN_RIGHTSPACE, d->rightspace->currentTitle());
+
+        d->rightspace->addedToController = true;
+        connect(d->rightspace, &WorkspaceWidget::workSpaceWidgeSwitched, this, [=](const QString &title) {
+            d->mainWindow->setDockHeaderName(WN_RIGHTSPACE, title);
+        });
+        connect(rightSpaceInfo.dockWidget, &QDockWidget::visibilityChanged, d->showRightspaceBtn, [=](bool visible){ d->showRightspaceBtn->setChecked(visible); });
+    } else if (!rightSpaceInfo.hiddenByManual) {
+        d->mainWindow->showWidget(WN_RIGHTSPACE);
+    }
+    d->currentDocks.append(WN_RIGHTSPACE);
+}
+
 bool Controller::hasContextWidget(const QString &title)
 {
     return d->contextWidgets.contains(title);
@@ -700,110 +789,6 @@ void Controller::switchContextWidget(const QString &title)
     }
 }
 
-void Controller::addChildMenu(AbstractMenu *abstractMenu)
-{
-    DMenu *inputMenu = abstractMenu->qMenu();
-    if (!d->mainWindow || !inputMenu)
-        return;
-
-    foreach (AbstractAction *action, abstractMenu->actionList()) {
-        if (action && action->hasShortCut()) {
-            registerActionShortCut(action);
-            addMenuShortCut(action->qAction());
-        }
-    }
-
-    // make the `helper` menu at the last
-    for (QAction *action : d->menu->actions()) {
-        if (action->text() == MWM_TOOLS) {
-            d->menu->insertMenu(action, inputMenu);
-            return;
-        }
-    }
-
-    // add menu to last
-    d->menu->addMenu(inputMenu);
-}
-
-void Controller::insertAction(const QString &menuName, const QString &beforActionName, AbstractAction *action)
-{
-    if (!action)
-        return;
-
-    QAction *inputAction = action->qAction();
-    if (!inputAction)
-        return;
-
-    if (action->hasShortCut())
-        registerActionShortCut(action);
-
-    for (QAction *qAction : d->mainWindow->menuBar()->actions()) {
-        if (qAction->text() == menuName) {
-            for (auto childAction : qAction->menu()->actions()) {
-                if (childAction->text() == beforActionName) {
-                    qAction->menu()->insertAction(childAction, inputAction);
-                    break;
-                } else if (qAction->text() == MWM_FILE
-                           && childAction->text() == MWMFA_QUIT) {
-                    qAction->menu()->insertAction(qAction, inputAction);
-                    break;
-                }
-            }
-        }
-    }
-}
-
-void Controller::addAction(const QString &menuName, AbstractAction *action)
-{
-    QAction *inputAction = action->qAction();
-    if (!inputAction)
-        return;
-
-    if (action->hasShortCut())
-        registerActionShortCut(action);
-
-    //防止与edit和debug界面的topToolBar快捷键冲突
-    if (menuName != MWM_DEBUG && menuName != MWM_BUILD)
-        addMenuShortCut(inputAction);
-
-    if (menuName == MWMFA_NEW_FILE_OR_PROJECT) {
-        for (QAction *qAction : d->menu->actions()) {
-            if (qAction->text() == MWM_BUILD) {
-                d->menu->insertAction(qAction, inputAction);
-                d->menu->insertSeparator(qAction);
-                return;
-            }
-        }
-    }
-
-    for (QAction *qAction : d->menu->actions()) {
-        if (qAction->text() == menuName) {
-            if (qAction->text() == MWM_FILE) {
-                auto endAction = *(qAction->menu()->actions().rbegin());
-                if (endAction->text() == MWMFA_QUIT) {
-                    return qAction->menu()->insertAction(endAction, inputAction);
-                }
-            }
-            qAction->menu()->addAction(inputAction);
-        }
-    }
-
-    if (!inputAction->parent())
-        inputAction->setParent(this);
-}
-
-void Controller::removeActions(const QString &menuName)
-{
-    for (QAction *qAction : d->mainWindow->menuBar()->actions()) {
-        if (qAction->text() == menuName) {
-            foreach (QAction *action, qAction->menu()->actions()) {
-                qAction->menu()->removeAction(action);
-            }
-            break;
-        }
-    }
-}
-
 void Controller::addWidgetToTopTool(AbstractWidget *abstractWidget, bool addSeparator, bool addToLeft, quint8 priority)
 {
     if (!abstractWidget)
@@ -848,28 +833,24 @@ void Controller::addWidgetToTopTool(AbstractWidget *abstractWidget, bool addSepa
     hlayout->insertWidget(index, widget);
 }
 
-void Controller::addTopToolItem(AbstractAction *action, bool addSeparator, quint8 priority)
+DToolButton *Controller::addTopToolItem(Command *action, bool addSeparator, quint8 priority)
 {
-    if (!action || !action->qAction())
-        return;
+    if (!action || !action->action())
+        return nullptr;
 
-    if (action->hasShortCut())
-        registerActionShortCut(action);
-
-    auto iconBtn = createIconButton(action->qAction());
+    auto iconBtn = createIconButton(action->action());
     addWidgetToTopTool(new AbstractWidget(iconBtn), addSeparator, true, priority);
+    return iconBtn;
 }
 
-void Controller::addTopToolItemToRight(AbstractAction *action, bool addSeparator, quint8 priority)
+DToolButton *Controller::addTopToolItemToRight(Command *action, bool addSeparator, quint8 priority)
 {
-    if (!action || !action->qAction())
-        return;
+    if (!action || !action->action())
+        return nullptr;
 
-    if (action->hasShortCut())
-        registerActionShortCut(action);
-
-    auto iconBtn = createIconButton(action->qAction());
+    auto iconBtn = createIconButton(action->action());
     addWidgetToTopTool(new AbstractWidget(iconBtn), addSeparator, false, priority);
+    return iconBtn;
 }
 
 void Controller::showTopToolBar()
@@ -906,8 +887,11 @@ void Controller::initMainWindow()
     qInfo() << __FUNCTION__;
     if (!d->mainWindow) {
         d->mainWindow = new MainWindow;
+        d->mainWindow->setObjectName("MainWindow");
         d->mainWindow->setMinimumSize(MW_MIN_WIDTH, MW_MIN_HEIGHT);
-        initMenu();
+        new ActionManager(this);
+        registerDefaultContainers();
+        registerDefaultActions();
 
         QString initFile = CustomPaths::user(CustomPaths::Configures) + "/mainwindow.ini";
         QFile file(initFile);
@@ -964,89 +948,6 @@ void Controller::initNavigationBar()
     vLayout->setContentsMargins(0, 0, 2, 0);
 }
 
-void Controller::initMenu()
-{
-    qInfo() << __FUNCTION__;
-    if (!d->mainWindow)
-        return;
-    if (!d->menu)
-        d->menu = new DMenu(d->mainWindow->titlebar());
-
-    createFileActions();
-    createBuildActions();
-    createDebugActions();
-
-    d->menu->addSeparator();
-
-    createHelpActions();
-    createToolsActions();
-
-    d->mainWindow->titlebar()->setMenu(d->menu);
-}
-
-void Controller::createHelpActions()
-{
-    auto helpMenu = new DMenu(MWM_HELP, d->menu);
-    d->menu->addMenu(helpMenu);
-
-    QAction *actionReportBug = new QAction(MWM_REPORT_BUG, helpMenu);
-    ActionManager::getInstance()->registerAction(actionReportBug, "Help.Report.Bug",
-                                                 MWM_REPORT_BUG, QKeySequence());
-    addMenuShortCut(actionReportBug);
-    helpMenu->addAction(actionReportBug);
-
-    QAction *actionHelpDoc = new QAction(MWM_HELP_DOCUMENTS, helpMenu);
-    ActionManager::getInstance()->registerAction(actionHelpDoc, "Help.Help.Documents",
-                                                 MWM_HELP_DOCUMENTS, QKeySequence());
-    helpMenu->addAction(actionHelpDoc);
-    addMenuShortCut(actionHelpDoc);
-
-    helpMenu->addSeparator();
-
-    QAction::connect(actionReportBug, &QAction::triggered, this, [=]() {
-        QDesktopServices::openUrl(QUrl("https://github.com/linuxdeepin/deepin-unioncode/issues"));
-    });
-    QAction::connect(actionHelpDoc, &QAction::triggered, this, [=]() {
-        QDesktopServices::openUrl(QUrl("https://uosdn.uniontech.com/#document2?dirid=656d40a9bd766615b0b02e5e"));
-    });
-}
-
-void Controller::createToolsActions()
-{
-    auto toolsMenu = new DMenu(MWM_TOOLS);
-    d->menu->addMenu(toolsMenu);
-}
-
-void Controller::createDebugActions()
-{
-    auto *debugMenu = new DMenu(MWM_DEBUG);
-    d->menu->addMenu(debugMenu);
-}
-
-void Controller::createBuildActions()
-{
-    auto *buildMenu = new DMenu(MWM_BUILD);
-    d->menu->addMenu(buildMenu);
-}
-
-void Controller::createFileActions()
-{
-    QAction *actionOpenFile = new QAction(MWMFA_OPEN_FILE);
-    ActionManager::getInstance()->registerAction(actionOpenFile, "File.Open.File",
-                                                 MWMFA_OPEN_FILE, QKeySequence(Qt::Modifier::CTRL | Qt::Key::Key_O));
-
-    QAction::connect(actionOpenFile, &QAction::triggered, this, &Controller::openFileDialog);
-    d->menu->addAction(actionOpenFile);
-    addMenuShortCut(actionOpenFile);
-
-    QAction *actionOpenProject = new QAction(MWMFA_OPEN_PROJECT);
-    d->menu->addAction(actionOpenProject);
-    QAction::connect(actionOpenProject, &QAction::triggered, this, [=]() {
-        auto prjService = dpfGetService(ProjectService);
-        prjService->openProject();
-    });
-}
-
 void Controller::initContextWidget()
 {
     if (!d->stackContextWidget)
@@ -1096,14 +997,14 @@ void Controller::initContextWidget()
     // add contextWidget after add centralWidget or it`s height is incorrect
     WidgetInfo info;
     info.name = WN_CONTEXTWIDGET;
-    info.widget = d->contextWidget;
+    info.setWidget(d->contextWidget);
     info.defaultPos = Position::Bottom;
     info.icon = QIcon::fromTheme("context_widget");
 
     if (d->statusBar) {
-        auto btn = createDockButton(info);
-        btn->setChecked(true);
-        d->statusBar->insertPermanentWidget(0, btn);
+        d->showContextBtn = createDockButton(info);
+        d->showContextBtn->setChecked(true);
+        d->statusBar->insertPermanentWidget(0, d->showContextBtn);
     }
 
     d->allWidgets.insert(WN_CONTEXTWIDGET, info);
@@ -1115,6 +1016,7 @@ void Controller::initStatusBar()
         return;
     d->statusBar = new WindowStatusBar(d->mainWindow);
     d->statusBar->hide();
+
     d->mainWindow->setStatusBar(d->statusBar);
 }
 
@@ -1127,12 +1029,37 @@ void Controller::initWorkspaceWidget()
 
     WidgetInfo info;
     info.name = WN_WORKSPACE;
-    info.widget = d->workspace;
+    info.setWidget(d->workspace);
     info.defaultPos = Position::Left;
     info.replace = true;
     d->allWidgets.insert(WN_WORKSPACE, info);
 }
 
+void Controller::initRightspaceWidget()
+{
+    if (d->rightspace)
+        return;
+
+    d->rightspace = new WorkspaceWidget(d->mainWindow);
+
+    WidgetInfo info;
+    info.name = WN_RIGHTSPACE;
+    info.setWidget(d->rightspace);
+    info.defaultPos = Position::Right;
+    info.replace = true;
+    info.icon = QIcon::fromTheme("uc_right_show");
+    d->allWidgets.insert(WN_RIGHTSPACE, info);
+
+    d->showRightspaceBtn = createDockButton(info);
+    d->showRightspaceBtn->setChecked(true);
+    auto scAction = new QAction(tr("Open rightspace"), d->rightspace);
+    auto cmd = ActionManager::instance()->registerAction(scAction, "Core.Open.Rightspace");
+    cmd->setDefaultKeySequence(QKeySequence(Qt::ALT + Qt::Key_L));
+    connect(scAction, &QAction::triggered, d->showRightspaceBtn, &DToolButton::clicked);
+
+    d->statusBar->insertPermanentWidget(0, d->showRightspaceBtn);
+}
+
 void Controller::initTopToolBar()
 {
     d->leftTopToolBar = new DWidget(d->mainWindow);
@@ -1178,19 +1105,103 @@ void Controller::initDocksManager()
     });
 }
 
-void Controller::addMenuShortCut(QAction *action, QKeySequence keySequence)
+void Controller::registerDefaultContainers()
 {
-    QKeySequence key = keySequence;
-    if (keySequence.isEmpty())
-        key = action->shortcut();
+    ActionManager::instance()->setContext({ C_GLOBAL });
+    auto titleContainer = ActionManager::instance()->createContainer(M_TITLEBAR);
+    titleContainer->appendGroup(G_FILE);
+    titleContainer->appendGroup(G_EDIT);
+    titleContainer->appendGroup(G_BUILD);
+    titleContainer->appendGroup(G_DEBUG);
+    titleContainer->appendGroup(G_TOOLS);
+    titleContainer->appendGroup(G_HELP);
 
-    QShortcut *shortCut = new QShortcut(key, d->mainWindow);
-    connect(action, &QAction::changed, this, [=]() {
-        if (action->shortcut() != shortCut->key())
-            shortCut->setKey(action->shortcut());
+    titleContainer->addSeparator(G_BUILD);
+    titleContainer->addSeparator(G_TOOLS);
+    d->mainWindow->titlebar()->setMenu(titleContainer->menu());
+
+    // file menu
+    auto fileContainer = ActionManager::instance()->createContainer(M_FILE);
+    fileContainer->menu()->setTitle(tr("&File"));
+    fileContainer->appendGroup(G_FILE_NEW);
+    fileContainer->appendGroup(G_FILE_OPEN);
+    fileContainer->appendGroup(G_FILE_CLOSE);
+    fileContainer->appendGroup(G_FILE_SAVE);
+
+    fileContainer->addSeparator(G_FILE_NEW);
+    fileContainer->addSeparator(G_FILE_OPEN);
+    fileContainer->addSeparator(G_FILE_CLOSE);
+    fileContainer->addSeparator(G_FILE_SAVE);
+    titleContainer->addMenu(fileContainer, G_FILE);
+
+    // edit menu
+    auto editContainer = ActionManager::instance()->createContainer(M_EDIT);
+    editContainer->menu()->setTitle(tr("&Edit"));
+    editContainer->appendGroup(G_EDIT_UNDOREDO);
+    editContainer->appendGroup(G_EDIT_COPYPASTE);
+    editContainer->appendGroup(G_EDIT_SELECTALL);
+    editContainer->appendGroup(G_EDIT_FIND);
+    editContainer->appendGroup(G_EDIT_OTHER);
+
+    editContainer->addSeparator(G_EDIT_COPYPASTE);
+    editContainer->addSeparator(G_EDIT_SELECTALL);
+    editContainer->addSeparator(G_EDIT_FIND);
+    titleContainer->addMenu(editContainer, G_EDIT);
+
+    // build menu
+    auto buildContainer = ActionManager::instance()->createContainer(M_BUILD);
+    buildContainer->menu()->setTitle(tr("&Build"));
+    titleContainer->addMenu(buildContainer, G_BUILD);
+
+    // debug menu
+    auto debugContainer = ActionManager::instance()->createContainer(M_DEBUG);
+    debugContainer->menu()->setTitle(tr("&Debug"));
+    titleContainer->addMenu(debugContainer, G_DEBUG);
+
+    // tools menu
+    auto toolsContainer = ActionManager::instance()->createContainer(M_TOOLS);
+    toolsContainer->menu()->setTitle(tr("&Tools"));
+    titleContainer->addMenu(toolsContainer, G_TOOLS);
+
+    // help menu
+    auto helpContainer = ActionManager::instance()->createContainer(M_HELP);
+    helpContainer->menu()->setTitle(tr("&Help"));
+    titleContainer->addMenu(helpContainer, G_HELP);
+}
+
+void Controller::registerDefaultActions()
+{
+    auto mFile = ActionManager::instance()->actionContainer(M_FILE);
+    auto mHelp = ActionManager::instance()->actionContainer(M_HELP);
+
+    // file actions
+    QAction *act = new QAction(tr("Open File"), mFile);
+    auto cmd = ActionManager::instance()->registerAction(act, A_OPEN_FILE);
+    cmd->setDefaultKeySequence(Qt::CTRL | Qt::Key_O);
+    mFile->addAction(cmd, G_FILE_OPEN);
+    connect(act, &QAction::triggered, this, &Controller::openFileDialog);
+
+    act = new QAction(tr("Open Project"), mFile);
+    cmd = ActionManager::instance()->registerAction(act, A_OPEN_PROJECT);
+    mFile->addAction(cmd, G_FILE_OPEN);
+    connect(act, &QAction::triggered, this, [=]() {
+        auto prjService = dpfGetService(ProjectService);
+        prjService->openProject();
     });
-    connect(shortCut, &QShortcut::activated, action, [=] {
-        action->trigger();
+
+    // help actions
+    act = new QAction(tr("Report Bug"), mHelp);
+    cmd = ActionManager::instance()->registerAction(act, A_REPORTBUG);
+    mHelp->addAction(cmd);
+    connect(act, &QAction::triggered, this, [=]() {
+        QDesktopServices::openUrl(QUrl("https://github.com/linuxdeepin/deepin-unioncode/issues"));
+    });
+
+    act = new QAction(tr("Help Documents"), mHelp);
+    cmd = ActionManager::instance()->registerAction(act, A_HELPDOC);
+    mHelp->addAction(cmd);
+    connect(act, &QAction::triggered, this, [=]() {
+        QDesktopServices::openUrl(QUrl("https://uosdn.uniontech.com/#document2?dirid=656d40a9bd766615b0b02e5e"));
     });
 }
 
@@ -1221,14 +1232,6 @@ void Controller::switchWorkspace(const QString &titleName)
         d->workspace->switchWidgetWorkspace(titleName);
 }
 
-void Controller::registerActionShortCut(AbstractAction *action)
-{
-    auto qAction = action->qAction();
-    if (!qAction)
-        return;
-    ActionManager::getInstance()->registerAction(qAction, action->id(), action->description(), action->keySequence());
-}
-
 void Controller::showWorkspace()
 {
     auto &workSpaceInfo = d->allWidgets[WN_WORKSPACE];
@@ -1272,6 +1275,10 @@ void Controller::showWorkspace()
 
         d->workspace->addedToController = true;
         bindWidgetToNavigation(WN_WORKSPACE, new AbstractAction(d->navigationActions[MWNA_EDIT]));
+
+        connect(d->workspace, &WorkspaceWidget::workSpaceWidgeSwitched, this, [=](const QString &title) {
+            d->mainWindow->setDockHeaderName(WN_WORKSPACE, title);
+        });
     } else {
         d->mainWindow->showWidget(WN_WORKSPACE);
     }
@@ -1285,15 +1292,15 @@ DToolButton *Controller::createIconButton(QAction *action)
     return iconBtn;
 }
 
-void Controller::removeTopToolItem(AbstractAction *action)
+void Controller::removeTopToolItem(Command *action)
 {
-    if (!action || !action->qAction())
+    if (!action || !action->action())
         return;
 
-    auto iconBtn = d->topToolBtn.value(action->qAction());
+    auto iconBtn = d->topToolBtn.value(action->action());
 
     delete iconBtn;
-    d->topToolBtn.remove(action->qAction());
+    d->topToolBtn.remove(action->action());
 }
 
 bool Controller::checkDocksManager()
@@ -1341,6 +1348,8 @@ DToolButton *Controller::createDockButton(const WidgetInfo &info)
     btn->setCheckable(true);
     connect(btn, &DToolButton::clicked, this, [=]() {
         auto &dockInfo = d->allWidgets[info.name];
+        if (!dockInfo.dockWidget)
+            return;
         if (dockInfo.dockWidget->isVisible()) {
             d->mainWindow->hideWidget(dockInfo.name);
             btn->setChecked(false);
@@ -1355,11 +1364,11 @@ DToolButton *Controller::createDockButton(const WidgetInfo &info)
     return btn;
 }
 
-void Controller::setTopToolItemVisible(AbstractAction *action, bool visible)
+void Controller::setTopToolItemVisible(Command *action, bool visible)
 {
-    if (!action || !action->qAction())
+    if (!action || !action->action())
         return;
 
-    auto iconBtn = d->topToolBtn.value(action->qAction());
+    auto iconBtn = d->topToolBtn.value(action->action());
     iconBtn->setVisible(visible);
 }
