@@ -17,6 +17,7 @@
 #include "dap/io.h"
 #include "dap/protocol.h"
 #include "dap/session.h"
+#include "dap/dap.h"
 #include "dap/network.h"
 
 #include <QDebug>
@@ -36,6 +37,7 @@ DebugSession::DebugSession(DebugModel *_model, QObject *parent)
       id(QUuid::createUuid().toString().toStdString()),
       model(_model)
 {
+    dap::initialize();
 }
 
 DebugSession::~DebugSession()
@@ -56,6 +58,7 @@ DebugSession::~DebugSession()
     }
     threads.clear();
 
+    dap::terminate();
     delete alertBox;
 }
 
@@ -509,6 +512,10 @@ void DebugSession::sendBreakpoints(const QString &sourcePath, dap::array<IBreakp
         SourceBreakpoint bt;
         bt.line = it.lineNumber;
         bt.condition = it.condition;
+        if (it.hitCondition.has_value()) {
+            bt.hitCondition = it.hitCondition;
+            model->removeBreakpoint(sourcePath, bt.line);
+        }
         breakpoints.push_back(bt);
     }
 
