@@ -62,7 +62,7 @@ GDBDebugger::GDBDebugger(QObject *parent)
 
 GDBDebugger::~GDBDebugger()
 {
-
+    delete d;
 }
 
 void GDBDebugger::init()
@@ -115,9 +115,11 @@ QString GDBDebugger::breakRemoveAll()
     return ("-break-delete");
 }
 
-QString GDBDebugger::breakInsert(const QString &path, const QString &condition)
+QString GDBDebugger::breakInsert(const QString &path, const QString &condition, bool isTemp)
 {
-    if (condition.isEmpty())
+    if (isTemp)
+        return QString{"-break-insert -t -f %1"}.arg(path);
+    else if (condition.isEmpty())
         return QString{"-break-insert -f %1"}.arg(path);
     else
         return QString{"-break-insert -c \"%1\" -f %2"}.arg(condition, path);
@@ -154,6 +156,11 @@ QString GDBDebugger::commandContinue()
     return ("-exec-continue");
 }
 
+QString GDBDebugger::commandReverseContinue()
+{
+    return ("reverse-continue");
+}
+
 QString GDBDebugger::commandNext()
 {
     return ("-exec-next");
@@ -169,6 +176,11 @@ QString GDBDebugger::commandFinish()
     return ("-exec-finish");
 }
 
+QString GDBDebugger::commandBack()
+{
+    return ("reverse-next");
+}
+
 QString GDBDebugger::threadInfo()
 {
     return ("-thread-info");
@@ -193,10 +205,14 @@ void GDBDebugger::updateBreakpoints(const QString &file, const QList<dap::Source
 {
     QSet<int> curLines;
     QMap<int, QString> lineConditionMap;
+    QSet<int> tempBreakPoints;
 
     for (auto bp : sourceBps) {
         QString condition = bp.condition.has_value() ? QString::fromStdString(bp.condition.value()) : "";
-        lineConditionMap.insert(bp.line, condition);
+        if (bp.hitCondition.has_value() && bp.hitCondition.value() == "1")
+            tempBreakPoints.insert(bp.line);
+        else
+            lineConditionMap.insert(bp.line, condition);
     }
 
     //remove canceled bp   and   bp which have condition (to update)
@@ -229,6 +245,12 @@ void GDBDebugger::updateBreakpoints(const QString &file, const QList<dap::Source
             }
         }
     }
+
+    // append temp breakpoints
+    for (const auto &bpLine : tempBreakPoints) {
+        auto filePath = file + ":" + QString::number(bpLine);
+        DebugManager::instance()->breakInsert(filePath, "", true);
+    }
 }
 
 void GDBDebugger::parseBreakPoint(const QVariant& var)
@@ -385,7 +407,18 @@ void GDBDebugger::handleOutputRecord(const QString &outputRecord)
         break;
     }
     case gdbmi::Record::RecordType::console:
+    {
+        if (record.message.contains("Core was generated by")) {
+            dap::StoppedEvent stoppedEvent;
+            stoppedEvent.reason = "step";
+            stoppedEvent.threadId = 1;
+            stoppedEvent.allThreadsStopped = true;
+            emit asyncStopped(stoppedEvent);
+
+            d->inferiorRunning.store(false);
+        }
         break;
+    }
     case gdbmi::Record::RecordType::target:
     {
         emit streamConsole(gdbmi::escapedText(record.message));
@@ -417,6 +450,10 @@ void GDBDebugger::parseNotifyData(gdbmi::Record &record)
         ctx.threadId = data.value("thread-id").toString();
         ctx.core = data.value("core").toInt();
         ctx.frame = gdbmi::Frame::parseMap(data.value("frame").toMap());
+        if (ctx.reason == gdbmi::AsyncContext::Reason::signalReceived) {
+            ctx.description = data.value("signal-name").toString();
+            ctx.text = data.value("signal-meaning").toString();
+        }
         d->inferiorRunning.store(false);
         sendStoppedNotify(ctx);
     } else if (record.message == "running") {
@@ -515,6 +552,10 @@ void GDBDebugger::sendStoppedNotify(const gdbmi::AsyncContext &ctx)
     stoppedEvent.threadId = ctx.threadId.toInt();
     stoppedEvent.allThreadsStopped = true;
     stoppedEvent.line = ctx.frame.line;
+    if (ctx.reason == gdbmi::AsyncContext::Reason::signalReceived) {
+        stoppedEvent.description = ctx.description.toStdString();
+        stoppedEvent.text = ctx.text.toStdString();
+    }
     dap::Source source;
     source.name = ctx.frame.file.toStdString();
     source.path = ctx.frame.fullpath.toStdString();
